<?PHP
###########################################################################
# 
# File:            taloLoggerGraph.inc (PHP)
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Usage:           To use the taloLoggerGraph log graphing package in 
#                  your website, first make sure that the files are
#                  located in your PHP-include-path.
#
#                  Secondly generate a configuration file for
#                  taloLoggerGraph. (see taloLoggerGraph.conf example
#                  configuration file and http://olammi.iki.fi/sw/taloLogger/
#                  for help).
#
#                  Utilise the sample index.php or add the following code to 
#                  your own web site code:
#
#                          include("taloLoggerGraph.src");
#
#                          taloLoggerGraph("taloLoggerGraph.conf");
#                      
#                  Replace the taloLoggerGraph.conf configuration file name with
#                  the name of the configuration file you generated. Remember
#                  to enclose the code in PHP-tags (see index.php for example).
# 
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.1b
#
# Date:            05.02.2013
#
# Functions:       taloLoggerGraph
#                   
# Description:     House automation data logger graphing application for PHP. 
# 
#                  The main function.
#
# Requirements:    PHP 5.1 or newer (www.php.net)
#
#                  Suitable PDO database support for PHP.
#
#                  JpGraph PHP-graphing tool (http://jpgraph.net/)
#
# Version history: ** 20.10.2010 v0.1a (Olli Lammi) **
#                  First beta version for testers. 
#
#                  ** 21.10.2010 v0.1b (Olli Lammi) **
#                  Small bug fixes. Database optimisations.
#
#                  ** 23.10.2010 v0.1c (Olli Lammi) **
#                  SQLite version.
#
#                  ** 03.11.2010 v0.1d (Olli Lammi) **
#                  Legend changes.
#
#                  ** 04.11.2010 v1.0a (Olli Lammi) **
#                  First version released to the public.
#
#                  ** 05.11.2010 v1.0b (Olli Lammi) **
#                  Bug fixes and support for non PDO 
#                  database connection for SQLite.
#
#                  ** 15.11.2010 v1.0c (Olli Lammi) **
#                  Support for IMAGE type graphs.
#
#                  ** 15.11.2010 v1.0d (Olli Lammi) **
#                  Added feature to hide the calendar.
#
#                  ** 16.11.2010 v1.0e (Olli Lammi) **
#                  Chart image size in parameter.
#
#                  ** 15.01.2013 v1.1a (Olli Lammi) **
#                  Support for taloLogger native DB structure.
#
#                  ** 05.02.2013 v1.1b (Olli Lammi) **
#                  Running last 24h of log instead of one day.
#
###########################################################################

include('taloLoggerGraph_functions.inc');

function taloLoggerGraph($confname) {
    $date = '';
    if (array_key_exists('date',$_REQUEST)) $date = $_REQUEST['date'];
    $period = 0; 
	if (array_key_exists('period',$_REQUEST)) $period = $_REQUEST['period'];
    
    $config = readConfigurationFile($confname);
    if ($config['stat'] != 0) {
        output_error($config['msg']);
        exit();
    }
    $config = $config['result'];

    $hidecal = 0;
    if (array_key_exists('HIDE_CALENDAR', $config['GENERAL'])) {
    	if (strtoupper($config['GENERAL']['HIDE_CALENDAR']) == 'TRUE') {
    		$hidecal = 1;
    	}
    }
    
    $DB_TYPE = $config["GENERAL"]["DB_TYPE"];
    if ($DB_TYPE == 'MYSQL') {
        $LOG_HOST = $config["GENERAL"]["MYSQLDB_HOST"];
        $LOG_USER = $config["GENERAL"]["MYSQLDB_USER"];
        $LOG_PASSWD = $config["GENERAL"]["MYSQLDB_PASSWD"];
        $LOG_DBNAME = $config["GENERAL"]["MYSQLDB_NAME"];
    }
    else if ($DB_TYPE == 'SQLITE') {
        $LOG_DBNAME = $config["GENERAL"]["SQLITEDB_NAME"];
    }
    else {
        output_error("ERROR: Invalid database type in configuration: " . $DB_TYPE);
        exit();        
    }
    
    $dbnative = 0;
    if (array_key_exists('DB_TALOLOGGER_SCHEMA_VERSION', $config['GENERAL'])) {
    	if ($config['GENERAL']['DB_TALOLOGGER_SCHEMA_VERSION'] == '1') {
    		$dbnative = 1;
    	}
    	else if ($config['GENERAL']['DB_TALOLOGGER_SCHEMA_VERSION'] == '0') {
    		$dbnative = 0;
    	}
    	else {
        	output_error("ERROR: Invalid database native schema version in configuration.");
        	exit();        
    	}
    }
        			
    if ($dbnative == 1) {
    	$config['GENERAL']['DB_TABLE'] = 'talo_data';
        $config['GENERAL']['DB_TIMECOL'] = 'time';
        $config['GENERAL']['DB_TALOLOGGER_SCHEMA_VERSION'] = 1;
    }
    else {
        $config['GENERAL']['DB_TALOLOGGER_SCHEMA_VERSION'] = 0;
    }
    
    $LOG_TABLE = $config["GENERAL"]["DB_TABLE"];
    $LOG_TIMECOL = $config["GENERAL"]["DB_TIMECOL"];
 
    $conn = null;
    if ($DB_TYPE == 'MYSQL') {
        $conn = new MySQLDBConnection($LOG_HOST, $LOG_DBNAME, $LOG_USER, $LOG_PASSWD);
    } 
    else if ($DB_TYPE == 'SQLITE') {
        $conn = new SQLiteDBConnection($LOG_DBNAME);
    }
           
    if (!$conn || !$conn->isOpen()) {
        output_error("ERROR: Unable to open/connect database.");
        exit();        
    }
  
    $today = strftime('%Y-%m-%d');
    $t_vuosi = strftime('%Y');
    $t_kuukausi = strftime('%m');
    $t_paiva = strftime('%d');

    $d_vuosi = $t_vuosi;
    $d_kuukausi = $t_kuukausi;
    $d_paiva = $t_paiva;
    $is_tanaan = 1;
    $is_full_day = 1;
    if ($date && preg_match("/^(?P<vuosi>\d+)-(?P<kuukausi>\d+)-(?P<paiva>\d+)$/", $date, $regmatch)) {
        $d_vuosi = $regmatch['vuosi'];
        $d_kuukausi = $regmatch['kuukausi'];
        $d_paiva = $regmatch['paiva'];
        $is_tanaan = 0;
    }

    if (!$hidecal) {
	    $vuodet = array();
	    if ($DB_TYPE == 'MYSQL') {
	        $sqlstmt = "SELECT YEAR(MIN(" . $LOG_TIMECOL . ")) AS ymin, " .
	                   "YEAR(MAX(" . $LOG_TIMECOL . ")) AS ymax FROM " .
	                   $LOG_TABLE;
	    }
	    else if ($DB_TYPE == 'SQLITE') {
	        $sqlstmt = "SELECT ymin, ymax FROM " .
	                   "(SELECT STRFTIME('%Y', DATETIME(" . $LOG_TIMECOL . ", 'localtime')) AS ymin FROM " . 
	                   $LOG_TABLE . " WHERE " . $LOG_TIMECOL . "=(SELECT MIN(". $LOG_TIMECOL .") FROM " . $LOG_TABLE. ") LIMIT 1), " .
	                   "(SELECT STRFTIME('%Y', DATETIME(" . $LOG_TIMECOL . ", 'localtime')) AS ymax FROM " . 
	                   $LOG_TABLE . " WHERE " . $LOG_TIMECOL . "=(SELECT MAX(". $LOG_TIMECOL .") FROM " . $LOG_TABLE. ") LIMIT 1)";
	    }
	    $conn->query($sqlstmt);
	    if($row = $conn->fetchRow()) {
	        $ymin = $row["ymin"];
	        $ymax = $row["ymax"];
	    }
	    else {
	        output_error("ERROR: No data in database.");
	        exit();        
	    }
	    $conn->freeResult();
	    
	    if ($ymin) $vuodet[] = $ymin;
	    if ($ymax - $ymin > 1) {
	        $y = $ymin + 1;
	        while ($y < $ymax) {
	            $sqlstmt = "SELECT " . $LOG_TIMECOL . 
	                       " FROM " . $LOG_TABLE . " WHERE " . 
	                       yearEhto($DB_TYPE, $LOG_TIMECOL, $y) . " LIMIT 1";
	            $conn->query($sqlstmt);
	            if ($conn->fetchRow()) {
	                $vuodet[] = $y;
	            }
	            $conn->freeResult();
	 
	            $y = $y + 1;
	        }
	    }
	    if ($ymax > $ymin) $vuodet[] = $ymax;
	
	    $kuukaudet = array();    
	    $kk = 1;
	    while ($kk <= 12) {
	        $sqlstmt = "SELECT " . $LOG_TIMECOL . 
	                   " FROM " . $LOG_TABLE . " WHERE " . 
	                   monthEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $kk) . " LIMIT 1";
	        $conn->query($sqlstmt);
	        if ($conn->fetchRow()) {
	            $kuukaudet[] = $kk;
	        }
	        $conn->freeResult();
	
	        $kk = $kk + 1;
	    }
	
	    $paivat = array();
	    if ($DB_TYPE == 'MYSQL') {
	        $sqlstmt = "SELECT DAY(MIN(" . $LOG_TIMECOL . ")) AS dmin, " .
	                   "DAY(MAX(" . $LOG_TIMECOL . ")) AS dmax FROM " .
	                   $LOG_TABLE . " WHERE " . 
	                   monthEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $d_kuukausi);
	    }
	    else if ($DB_TYPE == 'SQLITE') {           
	        $sqlstmt = "SELECT STRFTIME('%d', DATETIME(MIN(" . $LOG_TIMECOL . "), 'localtime')) AS dmin, " .
	                   "STRFTIME('%d', DATETIME(MAX(" . $LOG_TIMECOL . "), 'localtime')) AS dmax FROM " .
	                   $LOG_TABLE . " WHERE " . 
	                   monthEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $d_kuukausi);
	    }
	    $conn->query($sqlstmt);
	    if($row = $conn->fetchRow()) {
	        $dmin = $row["dmin"];
	        $dmax = $row["dmax"];
	    }
	    $conn->freeResult();
	
	    if ($dmin) $paivat[] = $dmin;
	    if ($dmax - $dmin > 1) {
	        $d = $dmin + 1;
	        while ($d < $dmax) {
	            $sqlstmt = "SELECT " . $LOG_TIMECOL . 
	                       " FROM " . $LOG_TABLE . " WHERE " . 
	                       dayEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $d_kuukausi, $d) . 
	                       " LIMIT 1";
	            $conn->query($sqlstmt);
	            if ($conn->fetchRow()) {
	                $paivat[] = $d;
	            }
	            $conn->freeResult();
	
	            $d = $d + 1;
	        }
	    }
	    if ($dmax > $dmin) $paivat[] = $dmax;
?>

<TABLE BORDER=1 ALIGN="center" WIDTH="100%">
<TR><TD ALIGN=center VALIGN=middle COLSPAN=3>
<A HREF="?date=&period=24">Last 24h</A>
<A HREF="?date=">Today</A>
</TD></TR>
<TR>
<TD WIDTH="33%" ALIGN=center VALIGN=middle>
<TABLE BORDER=0>
<TR>
<?PHP
	    $item = 0;
	    foreach ($vuodet as $vuosi) {
	        $modifier = '';
	        if ($vuosi == $d_vuosi) {
	            $modifier = " BGCOLOR=grey";
	        }
	
	        echo "<TD ALIGN=center" . $modifier . ">" 
	             . "<A HREF=\"?date=" . $vuosi . "-" . $d_kuukausi . "-" 
	             . $d_paiva . "\">" . $vuosi . "</A></TD>";
	        $item = ($item + 1) % 3;
	        if ($item == 0) echo "</TR><TR>";
	    }    
?>
</TR>
</TABLE>
</TD>
<TD WIDTH="33%" ALIGN=center VALIGN=middle>
<TABLE BORDER=0>
<TR>
<?PHP
	    $item = 0;
	    foreach ($kuukaudet as $kuukausi) {
	        $modifier = '';
	        if ($kuukausi == $d_kuukausi) {
	            $modifier = " BGCOLOR=grey";
	        }
	
	        echo "<TD ALIGN=center" . $modifier . ">" 
	             . "<A HREF=\"?date=" . $d_vuosi . "-" . $kuukausi . "-" 
	             . $d_paiva . "\">" . $kuukausi . "</A></TD>";
	        $item = ($item + 1) % 6;
	        if ($item == 0) echo "</TR><TR>";
	    }
?>
</TR>
</TABLE>
</TD>
<TD WIDTH="33%" ALIGN=center VALIGN=middle>
<TABLE BORDER=0>
<?PHP
	    if (count($paivat) > 0) {
	        echo "<TR>";
	        $minpaiva = $paivat[0];
	        $maxpaiva = $paivat[ count($paivat) - 1 ];
	        $vkonpaiva = (date('w', mktime(0, 0, 0, $d_kuukausi, $minpaiva, $d_vuosi)) + 6) % 7;
	
	        if ($vkonpaiva > 0) {
	            foreach( range(1, $vkonpaiva) as $i ) {
	                echo "<TD>&nbsp;</TD>";
	            }
	        }
	
	        foreach( range($minpaiva, $maxpaiva) as $paiva ) {
	            if ( in_array($paiva, $paivat) ) {
	                $modifier = '';
	                if ($paiva == $d_paiva) {
	                    $modifier = " BGCOLOR=grey";
	                }
	
	                echo "<TD ALIGN=center" . $modifier . ">" 
	                     . "<A HREF=\"?date=" . $d_vuosi . "-" . $d_kuukausi . "-" 
	                     . $paiva . "\">" . $paiva . "</A></TD>";
	            }
	            else {
	                echo "<TD>&nbsp;</TD>";
	            }
	
	            $vkonpaiva = ($vkonpaiva + 1) % 7;
	            if ($vkonpaiva == 0) {
	                echo "</TR><TR>";
	            }       
	        }
	
	        while ($vkonpaiva > 0) {
	            $vkonpaiva = ($vkonpaiva + 1) % 7;
	            echo "<TD>&nbsp;</TD>";
	        }
	
	        echo "</TR>\n";
	    }
?>

</TABLE>
</TD>
</TR>
</TABLE>


<?PHP
    }
    
	$has_data = 0;
	$starttime = 0;
	$endtime = 0;
	$calctime = time();
	if ($is_tanaan && $period > 0) {
		$endtime = nextFullHour($calctime);
		$starttime = $endtime - ($period * 3600);
		$is_full_day = 0;
	}
	else {		
	    $starttime = mktime(0, 0, 0, $d_kuukausi, $d_paiva, $d_vuosi);		
		$endtime = mktime(23, 59, 59, $d_kuukausi, $d_paiva, $d_vuosi) + 1;
	}

    $sqlstmt = "SELECT " . $LOG_TIMECOL .
               " FROM " . $LOG_TABLE . " WHERE ";
    if ($starttime > 0) {
    	$sqlstmt .= uxtimeEhto($DB_TYPE, $LOG_TIMECOL, $starttime, $endtime);
    }
    else {
    	$sqlstmt .= dayEhto($DB_TYPE, $LOG_TIMECOL, $d_vuosi, $d_kuukausi, $d_paiva); 
    }
	$sqlstmt .= " LIMIT 1";
    $conn->query($sqlstmt);
    if ( $conn->fetchRow() ) {
        $has_data = 1;
    }
    $conn->freeResult();

    if ($is_full_day) {
        echo "<P><DIV ALIGN=center>" . strftime('%d.%m.%Y', $starttime) . "<P>";
    }
    else {
    	echo "<P><DIV ALIGN=center>" . $period . " hours / " . getTimeStr($calctime) . "<P>";
    }
    
    if ($has_data > 0) {
        $ids = generateLoggerGraphs($DB_TYPE, $conn, $config, $is_tanaan, $is_full_day, $starttime, $endtime, $calctime);

        foreach($ids as $id) {
            echo '<IMG SRC="taloLoggerGraph_getchart.php?img=' . $id . '"><BR>';      
        }
    }
    else {
        echo "<I>No data in the database for requested date.</I>";
    }
    echo "</DIV>";

    $conn->close();
    $conn = null;
}
?>
