<?PHP 
###########################################################################
# 
# File:            taloLoggerGraph_tempfile.inc (PHP)
#
# License:         Donationware, see attached LICENSE file for more 
#                  information
#
# Author:          Olli Lammi (olammi@iki.fi)
#
# Version:         1.0a
#
# Date:            04.11.2010
#
# Functions:       getTempDir
#                  getImagePath
#                  getTempId
#                   
# Description:     House automation data logger graphing application for PHP. 
# 
#                  Functions to handle the generated images in the 
#                  system temporary directory.
#
# Requirements:    PHP 5 or newer (www.php.net)
#
# Version history: ** 20.10.2010 v0.1a (Olli Lammi) **
#                  First beta version for testers. 
#
#                  ** 04.11.2010 v1.0a (Olli Lammi) **
#                  First version released to the public.
#
###########################################################################

function getTempDir() {
    if (!function_exists('sys_get_temp_dir')) {
        # check environment variables.
        foreach (array('TMP', 'TEMP', 'TMPDIR') as $env_var) {
            if ($temp = getenv($env_var)) {
                return $temp;
            }
        }
        # test for a temp directory by having PHP create a temporary file.
        $temp = tempnam(__FILE__, '');
        if (file_exists($temp)) {
           unlink($temp);
           return dirname($temp);
        }
        # couldn't find a temp directory.
        return null;
    }
    else {
        return sys_get_temp_dir();
    }
}

function getImagePath($id) {
    return getTempDir() . DIRECTORY_SEPARATOR . 'tlchrt_' . $id . '.png';
}

function getTempId() {
    $id = rand();
    $i = 0;
    while ($i < 10) {
        if (!file_exists(getImagePath($id))) {
            return $id;
        }
        $i += 1;
    }

    return 0;
}

?>
